<h1 align="center">
  <!--img height="150" src="https://user-images.githubusercontent.com/24604927/147830756-70b99a0c-9f22-436d-bc78-6cc228491b35.png" /-->
  <img height="250" width="375" src="https://user-images.githubusercontent.com/24604927/147843028-ccdc5ee8-e3d8-4017-9923-f11e5ba37eb8.gif" />
  <br />
  <img src="https://img.shields.io/npms-io/final-score/c7" />
  <img src="https://img.shields.io/npms-io/maintenance-score/c7" />
  <img src="https://img.shields.io/npms-io/popularity-score/c7" />
  <img src="https://img.shields.io/npms-io/quality-score/c7" />
  <img src="https://img.shields.io/npm/dm/c7" />
  <br />
  <img src="https://img.shields.io/github/license/svasandani/c7" />
  <img src="https://img.shields.io/npm/v/c7?label=version" />
  <img src="https://img.shields.io/github/commit-activity/m/svasandani/c7" />
  <img src="https://img.shields.io/github/stars/svasandani/c7" />
  <br />
  <br />
  c7
  <br />
  <br />
</h1>
<p align="center">
  The <b>C</b>onfigurable <b>C</b>ommittable <b>C</b>ross-appli<b>C</b>ation <b>C</b>ustom <b>C</b>omponent <b>C</b>reator
</p>

<h2>
  Warning
  <img height="24px" src="https://user-images.githubusercontent.com/24604927/147856240-1eeb42e4-b703-4ea4-802b-10aa6ee11335.png" />
</h2>

This package is in an unreleased state. DO NOT USE PRE-1.X.X VERSIONS!

If you'd like to contribute, please see [CONTRIBUTING.md](CONTRIBUTING.md) and our [code of conduct](CODE_OF_CONDUCT.md).


## Quickstart
### Record

```bash
$ npx c7@latest record [<id> --<option1>=<value1> --<option2>=<value2> ...]
```

### Add

```bash
$ npx c7@latest add [<id> --<option1>=<value1> --<option2>=<value2> ...]
```

### Install

**Warning: We don't recommend installing `c7` globally. Use `npx` instead.**
```bash
$ npm install -g c7
```

## Do I need this?

Do you:
- Write lots of boilerplate? 
- Want the power of macros but with committable configuration files? 
- Want to standardize how components/routes/models/reusable bits are created in your repo? 
- Hate needing different component generators for different services?

**If you answered yes to at least one of the questions above, this package is for you!**

## Alright, I'm on board... how does it work?
For this follow-along, let's assume we're developing a React app with the following directory:

```
|-node_modules/
|-public/
|-src/
  |-components/
  |-hooks/
  |-pages/
  |-App.js
  |-App.cs
|-package.json
|-package-lock.json
```

### 1. Record yourself adding/editing files _(Optional, but recommended)_

I'll start by recording myself creating something reusable (it could be anything). Let's say I want to create a React component. I'll teach `c7` what to do by recording myself creating a `component` (but I could call it anything, like `route` or `model`). I'll also add a custom `name` parameter set to `Button` (this will make more sense later). To do that, I'll run this command:

```bash
$ c7 record component --name=Button
```

**Note: custom parameters must start with a lowercase letter. Uppercase parameters are specific to `c7`.**

It presents me with this prompt:

```bash
$ c7 record component --name=Button
Starting to record...
Make changes to your files, then press Enter to stop recording.
```

Great! I'll go ahead and **manually** add the `Button` component now.

#### Add `src/components/Button/Button.js`
```javascript
export const Button = () => {
  // TODO
}
```

#### Modify `src/components/index.js` (new lines indicated by comments)
```javascript
import { Button } from "./Button/Button.js"; // <- manually added this line
import { Input } from "./Input/Input.js";

export {
  Button, // <- manually added this line too
  Input,
};
```

Once I'm done, I'll go back to my terminal and hit any key. I'm now presented with this screen:

```bash
$ c7 record component --name=Button
Starting to record...
Make changes to your files, then press Enter to stop recording.
Stopped recording!

Calculating diffs...

CHANGES:
[ADDED]       "src/components/${name}/${name}.js"
[MODIFIED]    "src/components/index.js"
```

Now, `c7` creates the file `c7.json` and the directory `.c7` if they don't exist already. This config file and directory is where `c7` stores all the data it needs to recreate what we just did. 

It's smart enough to recognize that the path of the file we created contained our custom `name` parameter. If we were to define another custom parameter, for example `--basedir=src/components`, it would recognize that in the path too. It's also smart enough to understand how we modified the `index.js` file (we'll see how in a second).

I could technically create these artifacts (the configuration file and the directory) myself and still get the same result. That's the beauty of `c7`: it's configurable and controllable. It's just easier to do have them generated automatically. Now, let's get on to adding more components.

### 2. Add new components

I now want to add a new `Dropdown` component. It's as easy as running the following command:

```bash
$ c7 add component --name=Dropdown
```

Once I do, I'll see the following:

```bash
$ c7 add component --name=Dropdown
Applying diffs...

[ADDED]       `src/components/Dropdown/Dropdown.js`
[MODIFIED]    `src/components/index.js`
```

Awesome! Let's take a look at what it did automatically.

#### Added `src/components/Dropdown/Dropdown.js`
```javascript
export const Dropdown = () => {
  // TODO
}
```

#### Modified `src/components/index.js` (new lines indicated by comments)
```javascript
import { Button } from "./Button/Button.js";
import { Dropdown } from "./Dropdown/Dropdown.js"; // <- automatically added this line
import { Input } from "./Input/Input.js";

export {
  Button,
  Dropdown, // <- automatically added this line too
  Input,
};
```

Wow! It got the paths right, and it even parsed `index.js` to figure out that we were adding imports and exports. If you've modified a file that doesn't have a supported parser yet, it'll fall back to inserting or modifying line-by-line. 

### 3. Use all our saved time to figure out what else `c7` can do

`c7` isn't just for React components. Any time you need to programmatically add/modify file(s) in your codebase, `c7` can help. Take a look at a few examples below for inspiration:

#### Adding routes, handlers, and validators to a REST endpoint

**Note: `MatchCase` is a `c7` specific parameter that determines whether the exact case of custom parameters must be matched. Setting it to false allows `c7` to generate boilerplate for functions that don't use the exact case too (e.g. `validateUser`, etc.).**

```bash
$ c7 record model --name=user --MatchCase=false
Starting to record...
Make changes to your files, then press Enter to stop recording.
Stopped recording!

Calculating diffs...

CHANGES:
[ADDED]       `src/handlers/${name}.js`
[ADDED]       `src/validators/${name}.js`
[MODIFIED]    `src/routes.js`

Does this look right? (Y/n) _
```

#### Adding boilerplate types and error handlers to a CLI

```bash
$ c7 record submodule --subfunction=config --MatchCase=false
Watching for changes... (Press any key to end the recording.)
Stopped recording!

Calculating diffs...

CHANGES:
[ADDED]       `src/${subfunction}/functions.js`
[ADDED]       `src/${subfunction}/errors.js`
[ADDED]       `src/${subfunction}/types.js`
[ADDED]       `src/${subfunction}/test.js`

Does this look right? (Y/n) _
```

## `c7` configuration parameters

These can be configured either in `c7.json` or when running commands.

|Param|Type|Default|&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Description&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|
|-----|-----|-----|-----|
|`MatchCase`|Boolean|`true`|Determines whether to match the case of custom parameters. If false, will match any variant of every custom parameter in paths, new files, and modified files.|
|`MatchPath`|Boolean|`true`|Determines whether to match paths of created and modified files. If false, will always use the absolute path.|
|`AllowVars`|Boolean|`false`|Determines whether parameters can be passed as variables to other parameters. For example, if true, passing `--name=Param1 --test=[name]00` would set `name` to be `Param1` and `test` to be `Param100`.|

More to come soon!

## What's next?

Here's a list of planned and existing features:

|Feature|Description|Progress|
|-----|-----|-----|
|**Core**|Create directories and files from config files. Edit existing files based on text position.|`LIVE`|
|**Record**|Record component creation to autogenerate config files.|`IN PROGRESS`|
|**AST parsing**|Use AST diffs for more relevant edits to files.|`TODO`|
|**Plugin support**|Community plugin sypport to enhance functionality (for example presets for apps).|`TODO`|

If you have any suggestions or bugs, drop them in our [Issues](https://github.com/svasandani/c7/issues) page!

## Contributing

See [CONTRIBUTING.md](CONTRIBUTING.md).
