import { createHash } from "crypto";
import ignore from "ignore";
import { promises as fs } from "fs";
import path from "path";
const recursivelyUpdateNodes = async (node, ignores, tmpDir) => {
    const fullPath = path.join(process.cwd(), node.pathPrefix, node.identifier);
    const relativePath = path.relative(process.cwd(), fullPath);
    if (relativePath !== "" && ignores.ignores(relativePath))
        return;
    if ((await fs.lstat(fullPath)).isDirectory()) {
        node.type = "DIR";
        const subNodes = await fs.readdir(fullPath);
        subNodes.forEach((subNodeIdentifier) => {
            const subNode = new Node(node.rootPath, path.join(node.pathPrefix, node.identifier), subNodeIdentifier);
            node.nodes.push(subNode);
            recursivelyUpdateNodes(subNode, ignores, tmpDir);
        });
        node.hash = node.calculateHash("");
        const tmpPath = path.join(tmpDir, node.pathPrefix, node.identifier);
        fs.mkdir(path.join(tmpPath, `..`), { recursive: true });
        fs.mkdir(tmpPath);
    }
    else {
        node.type = "FILE";
        const buf = await fs.readFile(fullPath);
        const data = buf.toString();
        node.hash = node.calculateHash(data);
        const tmpPath = path.join(tmpDir, node.pathPrefix, node.identifier);
        fs.mkdir(path.join(tmpPath, `..`), { recursive: true });
        fs.writeFile(tmpPath, data);
    }
};
export class Node {
    constructor(rootPath, pathPrefix, identifier) {
        this.hash = "";
        this.rootPath = rootPath;
        this.pathPrefix = pathPrefix;
        this.identifier = identifier;
        this.type = "FILE";
        this.nodes = [];
    }
    toString() {
        return this.identifier + this.nodes.join();
    }
    calculateHash(withData) {
        return createHash("sha256")
            .update(`${this.toString()}${withData}`)
            .digest("hex");
    }
}
export class FileTree {
    constructor(rootDir) {
        this.rootDir = rootDir;
        this.rootNode = new Node(this.rootDir, "", "");
    }
    async parseFiles(tmpDir) {
        try {
            await fs.rm(path.join(tmpDir), { recursive: true, force: true });
            const ignores = ignore().add(`.git`);
            await fs.readFile(path.join(process.cwd(), `.gitignore`));
            const gitignore = await fs.readFile(path.join(process.cwd(), `.gitignore`));
            if (gitignore)
                ignores.add(gitignore.toString());
            recursivelyUpdateNodes(this.rootNode, ignores, tmpDir);
        }
        catch (err) { }
    }
    ;
}
